import * as TE from "fp-ts/lib/TaskEither";
import * as O from "fp-ts/lib/Option";
import type { EmailJob, TestJob } from "./job";
import type { D } from "../util";
import { perform } from "./email";

export enum TestType {
  EMAIL = "email",
  PING = "ping",
  HEALTHCHECK = "healthcheck",
  DNS = "dns"
}

export interface Schedule {
  every: D.Duration;
  jitter: D.Duration;
}

export interface Test {
  name: string;
  type: TestType;
  job: TestJob;
  schedule: Schedule;
}

export const parseTestType = (testType: string): O.Option<TestType> =>
  Object.values(TestType).includes(testType as TestType) ? O.some(testType as TestType) : O.none;

export const nextSchedule = (schedule: Schedule, date: Date) =>
  new Date(date.getTime() + schedule.every + Math.random() * schedule.jitter);

export const executorFor = (type: TestType, job: TestJob): (() => TE.TaskEither<Error, boolean>) => {
  switch (type) {
    case TestType.EMAIL:
      return () => perform(job as EmailJob);
  }
  return () => TE.right(true);
};
