import * as E from "fp-ts/Either";
import type { Publisher } from "./publisher";
import { readFileSync } from "fs";
import { TestType, type Test, type TestJob } from "./canary";
import * as D from "./util/duration";
import { pipe } from "fp-ts/lib/function";

export interface Config {
  result_publishers: Publisher[];
  dns: string[];
  http_timeout: string;
  tests: Test[];
}

export const transformDurations = (obj: any): E.Either<string, any> => {
  const transform = (o: any): E.Either<string, any> => {
    const entries = Object.entries(o);

    for (let [key, value] of entries) {
      if (key === "duration" && typeof value === "string") {
        return D.parse(value);
      } else if (typeof value === "object" && value !== null) {
        const result = transform(value);
        if (E.isLeft(result)) {
          return result;
        } else {
          o[key] = result.right;
        }
      }
    }

    return E.right(o);
  };

  return transform(obj);
};

export const readConfig = (filePath: string): E.Either<string, Config> =>
  pipe(readFileSync(filePath, "utf-8"), JSON.parse, transformDurations) as E.Either<string, Config>;
