import { toError } from "fp-ts/lib/Either";
import * as TE from "fp-ts/TaskEither";
import { publishDiscord, type DiscordPost } from "./discord";
import { publishNtfy, type NtfyPost } from "./ntfy";
import type { Logger } from "../util";
import { pipe } from "fp-ts/lib/function";

export enum PublisherType {
  DISCORD = "discord",
  NTFY = "ntfy",
  CONSOLE = "console"
}
export type PublisherPost = DiscordPost | NtfyPost | Logger;

export interface Publisher {
  publisherType: PublisherType;
  post: PublisherPost;
}

export const publish = (publisher: Publisher): ((message: string) => TE.TaskEither<Error, number>) => {
  switch (publisher.publisherType) {
    case PublisherType.DISCORD:
      return (message) => TE.tryCatch(() => publishDiscord(publisher.post as DiscordPost, message), toError);
    case PublisherType.NTFY:
      return (message) => TE.tryCatch(() => publishNtfy(publisher.post as NtfyPost, message), toError);
    case PublisherType.CONSOLE:
      return (message) =>
        pipe(
          (publisher.post as Logger).log("\n>=====<\n" + message + "\n>=====<"),
          TE.fromIO,
          TE.map(() => 200)
        );
    default:
      return (_message) => TE.left(new Error("unknown publisher type: " + publisher.publisherType));
  }
};
