import type { IO } from "fp-ts/lib/IO";

export interface Logger {
  log: (message: string) => IO<void>;
  error: (message: string) => IO<void>;
  addPrefix: (prefix: string) => Logger;
}

export class ConsoleLoggerI implements Logger {
  constructor(private readonly prefix = "") {}

  public log(message: string) {
    return () => console.log(new Date(), "[INFO]", this.prefix, message);
  }
  public error(message: string) {
    return () => console.error(new Date(), "[ERROR]", this.prefix, message);
  }
  public addPrefix(prefix: string): ConsoleLoggerI {
    return new ConsoleLoggerI(this.prefix + prefix);
  }
}

export const ConsoleLogger = new ConsoleLoggerI();
